# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import models, migrations
import django.db.models.deletion
import django.utils.timezone
import jsonfield.fields
import model_utils.fields


# Can't use the callable because the app registry is not ready yet.
# Really trusting users here... bad idea? probably.
DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL = getattr(settings, "DJSTRIPE_SUBSCRIBER_MODEL", settings.AUTH_USER_MODEL)

# Needed here for external apps that have added the DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL
# *not* in the '__first__' migration of the app, which results in:
# ValueError: Related model 'DJSTRIPE_SUBSCRIBER_MODEL' cannot be resolved
DJSTRIPE_SUBSCRIBER_MODEL_MIGRATION_DEPENDENCY = getattr(settings,
                                                         "DJSTRIPE_SUBSCRIBER_MODEL_MIGRATION_DEPENDENCY",
                                                         '__first__')

DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL_DEPENDENCY = migrations.swappable_dependency(DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL)

if DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL != settings.AUTH_USER_MODEL:
    DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL_DEPENDENCY = migrations.migration.SwappableTuple(
        (DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL.split(".", 1)[0], DJSTRIPE_SUBSCRIBER_MODEL_MIGRATION_DEPENDENCY),
        DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL
    )


class Migration(migrations.Migration):
    # not that squashmigrations generates [(b'djstripe', '0001_initial'),....
    # but for django in python3.X it needs to be [('djstripe', '0001_initial'),.... in order for the dependency
    # graph loader to remove the migrations this replaces
    replaces = [('djstripe', '0001_initial'), ('djstripe', '0002_auto_20150122_2000'),
                ('djstripe', '0003_auto_20150128_0800'), ('djstripe', '0004_auto_20150427_1609'),
                ('djstripe', '0005_charge_captured'), ('djstripe', '0006_auto_20150602_1934'),
                ('djstripe', '0007_auto_20150625_1243')]

    dependencies = [
        DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL_DEPENDENCY,
    ]

    operations = [
        migrations.CreateModel(
            name='Charge',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now,
                                                                verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now,
                                                                      verbose_name='modified', editable=False)),
                ('stripe_id', models.CharField(unique=True, max_length=50)),
                ('card_last_4', models.CharField(max_length=4, blank=True)),
                ('card_kind', models.CharField(max_length=50, blank=True)),
                ('amount', models.DecimalField(null=True, max_digits=7, decimal_places=2)),
                ('amount_refunded', models.DecimalField(null=True, max_digits=7, decimal_places=2)),
                ('description', models.TextField(blank=True)),
                ('paid', models.NullBooleanField()),
                ('disputed', models.NullBooleanField()),
                ('refunded', models.NullBooleanField()),
                ('fee', models.DecimalField(null=True, max_digits=7, decimal_places=2)),
                ('receipt_sent', models.BooleanField(default=False)),
                ('charge_created', models.DateTimeField(null=True, blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CurrentSubscription',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now,
                                                                verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now,
                                                                      verbose_name='modified', editable=False)),
                ('plan', models.CharField(max_length=100)),
                ('quantity', models.IntegerField()),
                ('start', models.DateTimeField()),
                ('status', models.CharField(max_length=25)),
                ('cancel_at_period_end', models.BooleanField(default=False)),
                ('canceled_at', models.DateTimeField(null=True, blank=True)),
                ('current_period_end', models.DateTimeField(null=True)),
                ('current_period_start', models.DateTimeField(null=True)),
                ('ended_at', models.DateTimeField(null=True, blank=True)),
                ('trial_end', models.DateTimeField(null=True, blank=True)),
                ('trial_start', models.DateTimeField(null=True, blank=True)),
                ('amount', models.DecimalField(max_digits=7, decimal_places=2)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now,
                                                                verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now,
                                                                      verbose_name='modified', editable=False)),
                ('stripe_id', models.CharField(unique=True, max_length=50)),
                ('card_fingerprint', models.CharField(max_length=200, blank=True)),
                ('card_last_4', models.CharField(max_length=4, blank=True)),
                ('card_kind', models.CharField(max_length=50, blank=True)),
                ('date_purged', models.DateTimeField(null=True, editable=False)),
                ('subscriber', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, to=DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now,
                                                                verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now,
                                                                      verbose_name='modified', editable=False)),
                ('stripe_id', models.CharField(unique=True, max_length=50)),
                ('kind', models.CharField(max_length=250)),
                ('livemode', models.BooleanField(default=False)),
                ('webhook_message', jsonfield.fields.JSONField(default=dict)),
                ('validated_message', jsonfield.fields.JSONField(null=True)),
                ('valid', models.NullBooleanField()),
                ('processed', models.BooleanField(default=False)),
                ('customer', models.ForeignKey(to='djstripe.Customer', on_delete=django.db.models.deletion.CASCADE, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EventProcessingException',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now,
                                                                verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now,
                                                                      verbose_name='modified', editable=False)),
                ('data', models.TextField()),
                ('message', models.CharField(max_length=500)),
                ('traceback', models.TextField()),
                ('event', models.ForeignKey(to='djstripe.Event', on_delete=django.db.models.deletion.CASCADE, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Invoice',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now,
                                                                verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now,
                                                                      verbose_name='modified', editable=False)),
                ('stripe_id', models.CharField(max_length=50)),
                ('attempted', models.NullBooleanField()),
                ('attempts', models.PositiveIntegerField(null=True)),
                ('closed', models.BooleanField(default=False)),
                ('paid', models.BooleanField(default=False)),
                ('period_end', models.DateTimeField()),
                ('period_start', models.DateTimeField()),
                ('subtotal', models.DecimalField(max_digits=7, decimal_places=2)),
                ('total', models.DecimalField(max_digits=7, decimal_places=2)),
                ('date', models.DateTimeField()),
                ('charge', models.CharField(max_length=50, blank=True)),
                ('customer', models.ForeignKey(related_name='invoices', on_delete=django.db.models.deletion.CASCADE, to='djstripe.Customer')),
            ],
            options={
                'ordering': ['-date'],
            },
        ),
        migrations.CreateModel(
            name='InvoiceItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now,
                                                                verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now,
                                                                      verbose_name='modified', editable=False)),
                ('stripe_id', models.CharField(max_length=50)),
                ('amount', models.DecimalField(max_digits=7, decimal_places=2)),
                ('currency', models.CharField(max_length=10)),
                ('period_start', models.DateTimeField()),
                ('period_end', models.DateTimeField()),
                ('proration', models.BooleanField(default=False)),
                ('line_type', models.CharField(max_length=50)),
                ('description', models.CharField(max_length=200, blank=True)),
                ('plan', models.CharField(max_length=100, null=True, blank=True)),
                ('quantity', models.IntegerField(null=True)),
                ('invoice', models.ForeignKey(related_name='items', on_delete=django.db.models.deletion.CASCADE, to='djstripe.Invoice')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Plan',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now,
                                                                verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now,
                                                                      verbose_name='modified', editable=False)),
                ('stripe_id', models.CharField(unique=True, max_length=50)),
                ('name', models.CharField(max_length=100)),
                ('currency', models.CharField(max_length=10, choices=[('usd', 'U.S. Dollars'),
                                                                      ('gbp', 'Pounds (GBP)'), ('eur', 'Euros')])),
                ('interval', models.CharField(max_length=10, verbose_name='Interval type',
                                              choices=[('week', 'Week'), ('month', 'Month'), ('year', 'Year')])),
                ('interval_count', models.IntegerField(default=1, null=True,
                                                       verbose_name='Intervals between charges')),
                ('amount', models.DecimalField(verbose_name='Amount (per period)', max_digits=7, decimal_places=2)),
                ('trial_period_days', models.IntegerField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Transfer',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now,
                                                                verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now,
                                                                      verbose_name='modified', editable=False)),
                ('stripe_id', models.CharField(unique=True, max_length=50)),
                ('amount', models.DecimalField(max_digits=7, decimal_places=2)),
                ('status', models.CharField(max_length=25)),
                ('date', models.DateTimeField()),
                ('description', models.TextField(null=True, blank=True)),
                ('adjustment_count', models.IntegerField()),
                ('adjustment_fees', models.DecimalField(max_digits=7, decimal_places=2)),
                ('adjustment_gross', models.DecimalField(max_digits=7, decimal_places=2)),
                ('charge_count', models.IntegerField()),
                ('charge_fees', models.DecimalField(max_digits=7, decimal_places=2)),
                ('charge_gross', models.DecimalField(max_digits=7, decimal_places=2)),
                ('collected_fee_count', models.IntegerField()),
                ('collected_fee_gross', models.DecimalField(max_digits=7, decimal_places=2)),
                ('net', models.DecimalField(max_digits=7, decimal_places=2)),
                ('refund_count', models.IntegerField()),
                ('refund_fees', models.DecimalField(max_digits=7, decimal_places=2)),
                ('refund_gross', models.DecimalField(max_digits=7, decimal_places=2)),
                ('validation_count', models.IntegerField()),
                ('validation_fees', models.DecimalField(max_digits=7, decimal_places=2)),
                ('event', models.ForeignKey(related_name='transfers', on_delete=django.db.models.deletion.CASCADE, to='djstripe.Event')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TransferChargeFee',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now,
                                                                verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now,
                                                                      verbose_name='modified', editable=False)),
                ('amount', models.DecimalField(max_digits=7, decimal_places=2)),
                ('application', models.TextField(null=True, blank=True)),
                ('description', models.TextField(null=True, blank=True)),
                ('kind', models.CharField(max_length=150)),
                ('transfer', models.ForeignKey(related_name='charge_fee_details', on_delete=django.db.models.deletion.CASCADE, to='djstripe.Transfer')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='currentsubscription',
            name='customer',
            field=models.OneToOneField(related_name='current_subscription', on_delete=django.db.models.deletion.CASCADE, null=True, to='djstripe.Customer'),
        ),
        migrations.AddField(
            model_name='charge',
            name='customer',
            field=models.ForeignKey(related_name='charges', on_delete=django.db.models.deletion.CASCADE, to='djstripe.Customer'),
        ),
        migrations.AddField(
            model_name='charge',
            name='invoice',
            field=models.ForeignKey(related_name='charges', on_delete=django.db.models.deletion.CASCADE, to='djstripe.Invoice', null=True),
        ),
        migrations.AlterField(
            model_name='event',
            name='webhook_message',
            field=jsonfield.fields.JSONField(),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='stripe_id',
            field=models.CharField(unique=True, max_length=50),
        ),
        migrations.AddField(
            model_name='charge',
            name='captured',
            field=models.NullBooleanField(),
        ),
        migrations.AddField(
            model_name='customer',
            name='card_exp_month',
            field=models.PositiveIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='customer',
            name='card_exp_year',
            field=models.PositiveIntegerField(null=True, blank=True),
        ),
    ]



from django.db import migrations
import djstripe.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0010_splitting_up_adds_from_alters'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charge',
            name='captured',
            field=djstripe.fields.StripeBooleanField(
                help_text='If the charge was created without capturing, this boolean represents whether or not it is \
                still uncaptured or has since been captured.', default=False),
        ),
    ]



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.webhooks.

  :synopsis: dj-stripe - Utils related to processing or registering for webhooks

.. moduleauthor:: Bill Huneke (@wahuneke)
.. moduleauthor:: Lee Skillen (@lskillen)

A model registers itself here if it wants to be in the list of processing
functions for a particular webhook. Each processor will have the ability
to modify the event object, access event data, and do what it needs to do

registrations are keyed by top-level event type (e.g. "invoice", "customer", etc)
Each registration entry is a list of processors
Each processor in these lists is a function to be called
The function signature is:
     <Event object> <event data dict> <event type> <event sub type>

The <event data dict> parameter should be a dict() structure, as received from Stripe
on webhook. This dict contains an 'object' member and also, sometimes, a 'previous_attributes'
member.

There is also a "global registry" which is just a list of processors (as defined above)

NOTE: global processors are called before other processors.
"""
from collections import defaultdict
import functools
import itertools

from django.utils import six

__all__ = ['handler', 'handler_all', 'call_handlers']


registrations = defaultdict(list)
registrations_global = list()

TEST_EVENT_ID = 'evt_00000000000000'


def handler(event_types):
    """
    Decorator that registers a function as a webhook handler.

    Functions can be registered for event types (e.g. 'customer') or
    fully qualified event sub-types (e.g. 'customer.subscription.deleted').

    If an event type is specified, the handler will receive callbacks for
    ALL webhook events of that type.  For example, if 'customer' is specified,
    the handler will receive events for 'customer.subscription.created',
    'customer.subscription.updated', etc.

    :param event_types: The event type(s) or sub-type(s) that should be handled.
    :type event_types: A sequence (`list`) or string (`str`/`unicode`).
    """
    if isinstance(event_types, six.string_types):
        event_types = [event_types]

    def decorator(func):
        for event_type in event_types:
            registrations[event_type].append(func)
        return func

    return decorator


def handler_all(func=None):
    """
    Decorator that registers a function as a webhook handler for ALL webhook events.

    Handles all webhooks regardless of event type or sub-type.
    """
    if not func:
        return functools.partial(handler_all)

    registrations_global.append(func)

    return func


def call_handlers(event, event_data, event_type, event_subtype):
    """
    Invoke all handlers for the provided event type/sub-type.

    The handlers are invoked in the following order:

        1. Global handlers
        2. Event type handlers
        3. Event sub-type handlers

    Handlers within each group are invoked in order of registration.

    :param event: The event model object.
    :type event: ``djstripe.models.Event``
    :param event_data: The raw data for the event.
    :type event_data: ``dict``
    :param event_type: The event type, e.g. 'customer'.
    :type event_type: string (``str``/``unicode``)
    :param event_subtype: The event sub-type, e.g. 'updated'.
    :type event_subtype: string (``str``/`unicode``)
    """
    chain = [registrations_global]

    # Build up a list of handlers with each qualified part of the event
    # type and subtype.  For example, "customer.subscription.created" creates:
    #   1. "customer"
    #   2. "customer.subscription"
    #   3. "customer.subscription.created"
    for index, _ in enumerate(event.parts):
        qualified_event_type = ".".join(event.parts[:(index + 1)])
        chain.append(registrations[qualified_event_type])

    for handler_func in itertools.chain(*chain):
        handler_func(event, event_data, event_type, event_subtype)



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.context_managers.

   :synopsis: dj-stripe Context Managers

.. moduleauthor:: Bill Huneke (@wahuneke)
.. moduleauthor:: Alex Kavanaugh (@akavanau)

"""

from contextlib import contextmanager


@contextmanager
def stripe_temporary_api_version(temp_version):
    """
    Temporarily replace the global api_version used in stripe API calls with the given value.

    The original value is restored as soon as context exits.
    """
    import stripe
    backup_version = stripe.api_version
    stripe.api_version = temp_version
    yield
    stripe.api_version = backup_version



"""
Migrations have been built for Django=>1.7 versions. Alternative migrations
for Django<1.7 users are provided with the ``south_migrations`` dir.

"""

SOUTH_ERROR_MESSAGE = """\n
For South support, customize the SOUTH_MIGRATION_MODULES setting like so:
    SOUTH_MIGRATION_MODULES = {
        'djstripe': 'djstripe.south_migrations',
    }
"""

try:
    from django.db import migrations  # noqa
except ImportError:
    from django.core.exceptions import ImproperlyConfigured
    raise ImproperlyConfigured(SOUTH_ERROR_MESSAGE)



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.middleware.

   :synopsis: dj-stripe middleware

   Refer to SubscriptionPaymentMiddleware docstring for more info.

.. moduleauthor:: @kavdev, @pydanny, @wahuneke
"""
from django.conf import settings
from django.core.urlresolvers import resolve
from django.shortcuts import redirect

import fnmatch

from .utils import subscriber_has_active_subscription
from .settings import subscriber_request_callback


DJSTRIPE_SUBSCRIPTION_REQUIRED_EXCEPTION_URLS = getattr(
    settings,
    "DJSTRIPE_SUBSCRIPTION_REQUIRED_EXCEPTION_URLS",
    ()
)

DJSTRIPE_SUBSCRIPTION_REDIRECT = getattr(
    settings,
    "DJSTRIPE_SUBSCRIPTION_REDIRECT",
    "djstripe:subscribe"
)


# So we don't have crazy long lines of code
EXEMPT = list(DJSTRIPE_SUBSCRIPTION_REQUIRED_EXCEPTION_URLS)
EXEMPT.append("[djstripe]")


class SubscriptionPaymentMiddleware(object):
    """
    Used to redirect users from subcription-locked request destinations.

    Rules:

        * "(app_name)" means everything from this app is exempt
        * "[namespace]" means everything with this name is exempt
        * "namespace:name" means this namespaced URL is exempt
        * "name" means this URL is exempt
        * The entire djstripe namespace is exempt
        * If settings.DEBUG is True, then django-debug-toolbar is exempt
        * A 'fn:' prefix means the rest of the URL is fnmatch'd.

    Example::

        DJSTRIPE_SUBSCRIPTION_REQUIRED_EXCEPTION_URLS = (
            "[blogs]",  # Anything in the blogs namespace
            "products:detail",  # A ProductDetail view you want shown to non-payers
            "home",  # Site homepage
            "fn:/accounts*",  # anything in the accounts/ URL path
        )
    """

    def process_request(self, request):
        """Check the subscriber's subscription status.

        Returns early if request does not outlined in this middleware's docstring.
        """
        if self.is_matching_rule(request):
            return

        return self.check_subscription(request)

    def is_matching_rule(self, request):
        """Check according to the rules defined in the class docstring."""
        # First, if in DEBUG mode and with django-debug-toolbar, we skip
        #   this entire process.
        if settings.DEBUG and request.path.startswith("/__debug__"):
            return True

        # Second we check against matches
        match = resolve(request.path, request.urlconf)
        if "({0})".format(match.app_name) in EXEMPT:
            return True

        if "[{0}]".format(match.namespace) in EXEMPT:
            return True

        if "{0}:{1}".format(match.namespace, match.url_name) in EXEMPT:
            return True

        if match.url_name in EXEMPT:
            return True

        # Third, we check wildcards:
        for exempt in [x for x in EXEMPT if x.startswith('fn:')]:
            exempt = exempt.replace('fn:', '')
            if fnmatch.fnmatch(request.path, exempt):
                return True

        return False

    def check_subscription(self, request):
        """Redirect to the subscribe page if the user lacks an active subscription."""
        subscriber = subscriber_request_callback(request)

        if not subscriber_has_active_subscription(subscriber):
            return redirect(DJSTRIPE_SUBSCRIPTION_REDIRECT)



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.event_handlers.

   :synopsis: dj-stripe - webhook event handlers for the various models

.. moduleauthor:: Bill Huneke (@wahuneke)
.. moduleauthor:: Alex Kavanaugh (@akavanau)
.. moduleauthor:: Lee Skillen (@lskillen)

Stripe docs for Events: https://stripe.com/docs/api#events
Stripe docs for Webhooks: https://stripe.com/docs/webhooks

TODO: Implement webhook event handlers for all the models that need to respond to webhook events.

NOTE: Event data is not guaranteed to be in the correct API version format. See #116.
      When writing a webhook handler, make sure to first re-retrieve the object you wish to
      process.

"""

from . import webhooks
from .models import Charge, Customer, Card, Subscription, Plan, Transfer, Invoice, InvoiceItem


@webhooks.handler_all
def customer_event_attach(event, event_data, event_type, event_subtype):
    """Make the related customer available on the event for all handlers to use.

    Does not create Customer objects.
    """
    event.customer = None
    crud_type = CrudType.determine(event_subtype, exact=True)

    if event_type == "customer" and crud_type.valid:
        customer_stripe_id = event_data["object"]["id"]
    else:
        customer_stripe_id = event_data["object"].get("customer", None)

    if customer_stripe_id:
        try:
            event.customer = Customer.objects.get(stripe_id=customer_stripe_id)
        except Customer.DoesNotExist:
            pass


@webhooks.handler("customer")
def customer_webhook_handler(event, event_data, event_type, event_subtype):
    """Handle updates to customer objects.

    First determines the crud_type and then handles the event if a customer exists locally.
    As customers are tied to local users, djstripe will not create customers that
    do not already exist locally.

    Docs and an example customer webhook response: https://stripe.com/docs/api#customer_object
    """
    crud_type = CrudType.determine(event_subtype, exact=True)
    if crud_type.valid and event.customer:
        # As customers are tied to local users, djstripe will not create
        # customers that do not already exist locally.
        _handle_crud_type_event(
            target_cls=Customer,
            event_data=event_data,
            event_subtype=event_subtype,
            crud_type=crud_type
        )


@webhooks.handler("customer.source")
def customer_source_webhook_handler(event, event_data, event_type, event_subtype):
    """Handle updates to customer payment-source objects.

    Docs: https://stripe.com/docs/api#customer_object-sources.
    """
    source_type = event_data["object"]["object"]

    # TODO: handle other types of sources (https://stripe.com/docs/api#customer_object-sources)
    if source_type == "card":
        _handle_crud_type_event(
            target_cls=Card,
            event_data=event_data,
            event_subtype=event_subtype,
            customer=event.customer
        )


@webhooks.handler("customer.subscription")
def customer_subscription_webhook_handler(event, event_data, event_type, event_subtype):
    """Handle updates to customer subscription objects.

    Docs an example subscription webhook response: https://stripe.com/docs/api#subscription_object
    """
    _handle_crud_type_event(
        target_cls=Subscription,
        event_data=event_data,
        event_subtype=event_subtype,
        customer=event.customer
    )


@webhooks.handler(["transfer", "charge", "invoice", "invoiceitem", "plan"])
def other_object_webhook_handler(event, event_data, event_type, event_subtype):
    """Handle updates to transfer, charge, invoice, invoiceitem and plan objects.

    Docs for:
    - charge: https://stripe.com/docs/api#charges
    - invoice: https://stripe.com/docs/api#invoices
    - invoiceitem: https://stripe.com/docs/api#invoiceitems
    - plan: https://stripe.com/docs/api#plans
    """
    target_cls = {
        "charge": Charge,
        "invoice": Invoice,
        "invoiceitem": InvoiceItem,
        "plan": Plan,
        "transfer": Transfer
    }.get(event_type)

    _handle_crud_type_event(
        target_cls=target_cls,
        event_data=event_data,
        event_subtype=event_subtype,
        customer=event.customer
    )


#
# Helpers
#

class CrudType(object):
    """Helper object to determine CRUD-like event state."""

    created = False
    updated = False
    deleted = False

    def __init__(self, **kwargs):
        """Set attributes."""
        for k, v in kwargs.items():
            setattr(self, k, v)

    @property
    def valid(self):
        """Return True if this is a CRUD-like event."""
        return self.created or self.updated or self.deleted

    @classmethod
    def determine(cls, event_subtype, exact=False):
        """
        Determine if the event subtype is a crud_type (without the 'R') event.

        :param event_subtype: The event subtype to examine.
        :type event_subtype: string (``str``/`unicode``)
        :param exact: If True, match crud_type to event subtype string exactly.
        :param type: ``bool``
        :returns: The CrudType state object.
        :rtype: ``CrudType``
        """
        def check(crud_type_event):
            if exact:
                return event_subtype == crud_type_event
            else:
                return event_subtype.endswith(crud_type_event)

        created = updated = deleted = False

        if check("updated"):
            updated = True
        elif check("created"):
            created = True
        elif check("deleted"):
            deleted = True

        return cls(created=created, updated=updated, deleted=deleted)


def _handle_crud_type_event(target_cls, event_data, event_subtype, stripe_id=None, customer=None, crud_type=None):
    """
    Helper to process crud_type-like events for objects.

    Non-deletes (creates, updates and "anything else" events) are treated as
    update_or_create events - The object will be retrieved locally, then it is
    synchronised with the Stripe API for parity.

    Deletes only occur for delete events and cause the object to be deleted
    from the local database, if it existed.  If it doesn't exist then it is
    ignored (but the event processing still succeeds).

    :param target_cls: The djstripe model being handled.
    :type: ``djstripe.stripe_objects.StripeObject``
    :param event_data: The event object data received from the Stripe API.
    :param event_subtype: The event subtype string.
    :param stripe_id: The object Stripe ID - If not provided then this is
    retrieved from the event object data by "object.id" key.
    :param customer: The customer object which is passed on object creation.
    :param crud_type: The CrudType object - If not provided it is determined
    based on the event subtype string.
    :returns: The object (if any) and the event CrudType.
    :rtype: ``tuple(obj, CrudType)``
    """
    crud_type = crud_type or CrudType.determine(event_subtype)
    stripe_id = stripe_id or event_data["object"]["id"]
    obj = None

    if crud_type.deleted:
        try:
            obj = target_cls.objects.get(stripe_id=stripe_id)
            obj.delete()
        except target_cls.DoesNotExist:
            pass
    else:
        # Any other event type (creates, updates, etc.)
        kwargs = {"stripe_id": stripe_id}
        if customer:
            kwargs["customer"] = customer
        data = target_cls(**kwargs).api_retrieve()
        obj = target_cls.sync_from_stripe_data(data)

    return obj, crud_type



# -*- coding: utf-8 -*-
"""
.. module:: dj-stripe.contrib.rest_framework.urls.

    :synopsis: URL routes for the dj-stripe REST API.

.. moduleauthor:: Philippe Luickx (@philippeluickx)

Wire this into the root URLConf this way::

    url(
        r'^api/v1/stripe/',
        include('djstripe.contrib.rest_framework.urls', namespace="rest_djstripe")
    ),
    # url can be changed
    # Call to 'djstripe.contrib.rest_framework.urls' and 'namespace' must stay as is

"""

from __future__ import unicode_literals
from django.conf.urls import url

from . import views


urlpatterns = [

    # REST api
    url(
        r"^subscription/$",
        views.SubscriptionRestView.as_view(),
        name="subscription"
    ),

]



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.exceptions.

   :synopsis: dj-stripe Exceptions.

.. moduleauthor:: @kavdev

"""
from __future__ import unicode_literals


class MultipleSubscriptionException(Exception):
    """Raised when a Customer has multiple Subscriptions and only one is expected."""

    pass


class StripeObjectManipulationException(Exception):
    """Raised when an attempt to manipulate a non-standalone stripe object is made not through its parent object."""

    pass


class CustomerDoesNotExistLocallyException(Exception):
    """Raised when a user tries to perform an action on a Customer that does not exist locally."""

    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0003_auto_20150128_0800'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='webhook_message',
            field=jsonfield.fields.JSONField(),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='stripe_id',
            field=models.CharField(max_length=50, unique=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-28 00:48
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
from django.db.models.deletion import SET_NULL


DJSTRIPE_SUBSCRIBER_MODEL = getattr(settings, "DJSTRIPE_SUBSCRIBER_MODEL", settings.AUTH_USER_MODEL)


def on_subscriber_delete_purge_customers(collector, field, sub_objs, using):
    """ Ensure that all customers attached to subscriber are purged on deletion. """
    for obj in sub_objs:
        obj.purge()

    SET_NULL(collector, field, sub_objs, using)



class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0018_field_docs'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='subscriber',
            field=models.OneToOneField(null=True, on_delete=on_subscriber_delete_purge_customers, to=DJSTRIPE_SUBSCRIBER_MODEL),
        ),
    ]



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.fields.

   :synopsis: dj-stripe Custom Field Definitions

.. moduleauthor:: Bill Huneke (@wahuneke)
"""

import decimal

from django.core.exceptions import ImproperlyConfigured, FieldError
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models

from jsonfield import JSONField

from .utils import dict_nested_accessor, convert_tstamp


class StripeFieldMixin(object):
    """
    Custom fields for all Stripe data.

    This allows keeping track of which database fields are suitable for
    sending to or receiving from Stripe. Also, allows a few handy extra parameters.
    """

    # Used if the name at stripe is different from the name in our database
    # Include a . in name if value is nested in dict in Stripe's object
    # (e.g.  stripe_name = "data.id"  -->  obj["data"]["id"])
    stripe_name = None

    # If stripe_name is None, this can also be used to specify a nested value, but
    # the final value is assumed to be the database field name
    # (e.g.    nested_name = "data"    -->  obj["data"][db_field_name]
    nested_name = None

    # This indicates that this field will always appear in a stripe object. It will be
    # an Exception if we try to parse a stripe object that does not include this field
    # in the data. If set to False then null=True attribute will be automatically set
    stripe_required = True

    # If a field was populated in previous API versions but we don't want to drop the old
    # data for some reason, mark it as deprecated. This will make sure we never try to send
    # it to Stripe or expect in Stripe data received
    # This setting automatically implies Null=True
    deprecated = False

    def __init__(self, *args, **kwargs):
        """
        Assign class instance variables based on kwargs.

        Assign extra class instance variables if stripe_required is defined or
        if deprecated is defined.
        """
        self.stripe_name = kwargs.pop('stripe_name', self.stripe_name)
        self.nested_name = kwargs.pop('nested_name', self.nested_name)
        self.stripe_required = kwargs.pop('stripe_required', self.stripe_required)
        self.deprecated = kwargs.pop('deprecated', self.deprecated)
        if not self.stripe_required:
            kwargs["null"] = True

        if self.deprecated:
            kwargs["null"] = True
            kwargs["default"] = None
        super(StripeFieldMixin, self).__init__(*args, **kwargs)

    def stripe_to_db(self, data):
        """Try converting stripe fields to defined database fields."""
        if not self.deprecated:
            try:
                if self.stripe_name:
                    result = dict_nested_accessor(data, self.stripe_name)
                elif self.nested_name:
                    result = dict_nested_accessor(data, self.nested_name + "." + self.name)
                else:
                    result = data[self.name]
            except (KeyError, TypeError):
                if self.stripe_required:
                    model_name = self.model._meta.object_name if hasattr(self, "model") else ""
                    raise FieldError("Required stripe field '{field_name}' was not"
                                     " provided in {model_name} data object.".format(field_name=self.name,
                                                                                     model_name=model_name))
                else:
                    result = None

            return result


class StripePercentField(StripeFieldMixin, models.DecimalField):
    """A field used to define a percent according to djstripe logic."""

    def __init__(self, *args, **kwargs):
        """Assign default args to this field."""
        defaults = {
            'decimal_places': 2,
            'max_digits': 5,
            'validators': [MinValueValidator(1.00), MaxValueValidator(100.00)]
        }
        defaults.update(kwargs)
        super(StripePercentField, self).__init__(*args, **defaults)


class StripeCurrencyField(StripeFieldMixin, models.DecimalField):
    """
    A field used to define currency according to djstripe logic.

    Stripe is always in cents. djstripe stores everything in dollars.
    """

    def __init__(self, *args, **kwargs):
        """Assign default args to this field."""
        defaults = {
            'decimal_places': 2,
            'max_digits': 7,
        }
        defaults.update(kwargs)
        super(StripeCurrencyField, self).__init__(*args, **defaults)

    def stripe_to_db(self, data):
        """Convert the raw value to decimal representation."""
        val = super(StripeCurrencyField, self).stripe_to_db(data)

        # Note: 0 is a possible return value, which is 'falseish'
        if val is not None:
            return val / decimal.Decimal("100")


class StripeBooleanField(StripeFieldMixin, models.BooleanField):
    """A field used to define a boolean value according to djstripe logic."""

    def __init__(self, *args, **kwargs):
        """Throw an error when a user tries to deprecate."""
        if kwargs.get("deprecated", False):
            raise ImproperlyConfigured("Boolean field cannot be deprecated. Change field type to "
                                       "StripeNullBooleanField")
        super(StripeBooleanField, self).__init__(*args, **kwargs)


class StripeNullBooleanField(StripeFieldMixin, models.NullBooleanField):
    """A field used to define a NullBooleanField value according to djstripe logic."""

    pass


class StripeCharField(StripeFieldMixin, models.CharField):
    """A field used to define a CharField value according to djstripe logic."""

    pass


class StripeIdField(StripeCharField):
    """A field with enough space to hold any stripe ID."""

    def __init__(self, *args, **kwargs):
        """
        Assign default args to this field.

        As per: https://stripe.com/docs/upgrades
        You can safely assume object IDs we generate will never exceed 255
        characters, but you should be able to handle IDs of up to that
        length.
        """
        defaults = {
            'max_length': 255,
            'blank': False,
            'null': False,
        }
        defaults.update(kwargs)
        super(StripeIdField, self).__init__(*args, **defaults)


class StripeTextField(StripeFieldMixin, models.TextField):
    """A field used to define a TextField value according to djstripe logic."""

    pass


class StripeDateTimeField(StripeFieldMixin, models.DateTimeField):
    """A field used to define a DateTimeField value according to djstripe logic."""

    def stripe_to_db(self, data):
        """Convert the raw timestamp value to a DateTime representation."""
        val = super(StripeDateTimeField, self).stripe_to_db(data)

        # Note: 0 is a possible return value, which is 'falseish'
        if val is not None:
            return convert_tstamp(val)


class StripeIntegerField(StripeFieldMixin, models.IntegerField):
    """A field used to define a IntegerField value according to djstripe logic."""

    pass


class StripeJSONField(StripeFieldMixin, JSONField):
    """A field used to define a JSONField value according to djstripe logic."""

    pass



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.forms.

   :synopsis: dj-stripe Forms.

.. moduleauthor:: Daniel Greenfeld (@pydanny)

"""

from django import forms

from .models import Plan


class PlanForm(forms.Form):
    """A form used when creating a Plan."""

    plan = forms.ModelChoiceField(queryset=Plan.objects.all())


class CancelSubscriptionForm(forms.Form):
    """A form used when canceling a Subscription."""

    pass



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.settings.

   :synopsis: dj-stripe settings

.. moduleauthor:: @kavdev, @pydanny, @lskillen, and @chrissmejia
"""
from __future__ import unicode_literals

import sys

from django.apps import apps as django_apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils import six
from django.utils.module_loading import import_string

PY3 = sys.version > "3"


def get_callback_function(setting_name, default=None):
    """
    Resolve a callback function based on a setting name.

    If the setting value isn't set, default is returned.  If the setting value
    is already a callable function, that value is used - If the setting value
    is a string, an attempt is made to import it.  Anything else will result in
    a failed import causing ImportError to be raised.

    :param setting_name: The name of the setting to resolve a callback from.
    :type setting_name: string (``str``/``unicode``)
    :param default: The default to return if setting isn't populated.
    :type default: ``bool``
    :returns: The resolved callback function (if any).
    :type: ``callable``
    """
    func = getattr(settings, setting_name, None)
    if not func:
        return default

    if callable(func):
        return func

    if isinstance(func, six.string_types):
        func = import_string(func)

    if not callable(func):
        raise ImproperlyConfigured("{name} must be callable.".format(name=setting_name))

    return func


subscriber_request_callback = get_callback_function("DJSTRIPE_SUBSCRIBER_MODEL_REQUEST_CALLBACK",
                                                    default=(lambda request: request.user))

INVOICE_FROM_EMAIL = getattr(settings, "DJSTRIPE_INVOICE_FROM_EMAIL", "billing@example.com")
PAYMENTS_PLANS = getattr(settings, "DJSTRIPE_PLANS", {})
PLAN_HIERARCHY = getattr(settings, "DJSTRIPE_PLAN_HIERARCHY", {})

PASSWORD_INPUT_RENDER_VALUE = getattr(settings, 'DJSTRIPE_PASSWORD_INPUT_RENDER_VALUE', False)
PASSWORD_MIN_LENGTH = getattr(settings, 'DJSTRIPE_PASSWORD_MIN_LENGTH', 6)

PRORATION_POLICY = getattr(settings, 'DJSTRIPE_PRORATION_POLICY', False)
PRORATION_POLICY_FOR_UPGRADES = getattr(settings, 'DJSTRIPE_PRORATION_POLICY_FOR_UPGRADES', False)
CANCELLATION_AT_PERIOD_END = not getattr(settings, 'DJSTRIPE_PRORATION_POLICY', False)

SEND_INVOICE_RECEIPT_EMAILS = getattr(settings, "DJSTRIPE_SEND_INVOICE_RECEIPT_EMAILS", True)
CURRENCIES = getattr(settings, "DJSTRIPE_CURRENCIES", (
    ('usd', 'U.S. Dollars',),
    ('gbp', 'Pounds (GBP)',),
    ('eur', 'Euros',))
)

DEFAULT_PLAN = getattr(settings, "DJSTRIPE_DEFAULT_PLAN", None)

# Try to find the new settings variable first. If that fails, revert to the
# old variable.
trial_period_for_subscriber_callback = (
    get_callback_function("DJSTRIPE_TRIAL_PERIOD_FOR_SUBSCRIBER_CALLBACK") or
    get_callback_function("DJSTRIPE_TRIAL_PERIOD_FOR_USER_CALLBACK"))

DJSTRIPE_WEBHOOK_URL = getattr(settings, "DJSTRIPE_WEBHOOK_URL", r"^webhook/$")

# Webhook event callbacks allow an application to take control of what happens
# when an event from Stripe is received.  One suggestion is to put the event
# onto a task queue (such as celery) for asynchronous processing.
WEBHOOK_EVENT_CALLBACK = get_callback_function("DJSTRIPE_WEBHOOK_EVENT_CALLBACK")


def get_subscriber_model_string():
    """Get the configured subscriber model as a module path string."""
    return getattr(settings, "DJSTRIPE_SUBSCRIBER_MODEL", settings.AUTH_USER_MODEL)


def get_subscriber_model():
    """
    Attempt to pull settings.DJSTRIPE_SUBSCRIBER_MODEL.

    Users have the option of specifying a custom subscriber model via the
    DJSTRIPE_SUBSCRIBER_MODEL setting.

    This methods falls back to AUTH_USER_MODEL if DJSTRIPE_SUBSCRIBER_MODEL is not set.

    Returns the subscriber model that is active in this project.
    """
    model_name = get_subscriber_model_string()

    # Attempt a Django 1.7 app lookup
    try:
        subscriber_model = django_apps.get_model(model_name)
    except ValueError:
        raise ImproperlyConfigured("DJSTRIPE_SUBSCRIBER_MODEL must be of the form 'app_label.model_name'.")
    except LookupError:
        raise ImproperlyConfigured("DJSTRIPE_SUBSCRIBER_MODEL refers to model '{model}' "
                                   "that has not been installed.".format(model=model_name))

    if (("email" not in [field_.name for field_ in subscriber_model._meta.get_fields()]) and
            not hasattr(subscriber_model, 'email')):
        raise ImproperlyConfigured("DJSTRIPE_SUBSCRIBER_MODEL must have an email attribute.")

    if model_name != settings.AUTH_USER_MODEL:
        # Custom user model detected. Make sure the callback is configured.
        func = get_callback_function("DJSTRIPE_SUBSCRIBER_MODEL_REQUEST_CALLBACK")
        if not func:
            raise ImproperlyConfigured(
                "DJSTRIPE_SUBSCRIBER_MODEL_REQUEST_CALLBACK must be implemented "
                "if a DJSTRIPE_SUBSCRIBER_MODEL is defined.")

    return subscriber_model



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.decorators.

   :synopsis: dj-stripe Decorators.

.. moduleauthor:: @pydanny, and @audreyr

"""
from __future__ import unicode_literals

from functools import wraps

from django.utils.decorators import available_attrs
from django.shortcuts import redirect

from .utils import subscriber_has_active_subscription
from .settings import subscriber_request_callback


def subscriber_passes_pay_test(test_func, plan=None, pay_page="djstripe:subscribe"):
    """
    Decorator for views that checks the subscriber passes the given test for a "Paid Feature".

    Redirects to `pay_page` if necessary. The test should be a callable
    that takes the subscriber object and returns True if the subscriber passes.
    """
    def decorator(view_func):
        @wraps(view_func, assigned=available_attrs(view_func))
        def _wrapped_view(request, *args, **kwargs):
            if test_func(subscriber_request_callback(request), plan):
                return view_func(request, *args, **kwargs)

            return redirect(pay_page)
        return _wrapped_view
    return decorator


def subscription_payment_required(function=None, plan=None, pay_page="djstripe:subscribe"):
    """
    Decorator for views that require subscription payment.

    Redirects to `pay_page` if necessary.
    """
    actual_decorator = subscriber_passes_pay_test(
        subscriber_has_active_subscription,
        plan=plan,
        pay_page=pay_page,
    )
    if function:
        return actual_decorator(function)
    return actual_decorator



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0006_auto_20150602_1934'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='card_exp_month',
            field=models.PositiveIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='customer',
            name='card_exp_year',
            field=models.PositiveIntegerField(null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.templatetags.djstripe_tags.

   :synopsis: dj-stripe - Custom tags to be used in templates.

.. moduleauthor:: @kavdev, @pydanny, @chrissmejia, @audreyr
"""
from __future__ import division

from django.template import Library
from .. import settings as djstripe_settings

register = Library()


@register.filter
def djdiv(value, arg):
    """
    Divide the value by the arg, using Python 3-style division that returns floats.

    If bad values are passed in, return the empty string.
    """
    try:
        return value / arg
    except (ValueError, TypeError):
        try:
            return value / arg
        except Exception:
            return ''


division.is_safe = False


@register.filter(name='djstripe_plan_level')
def djstripe_plan_level(name):
    """
    Add support to levels over plans, then you can have different kind of plans with the level same access.

    Use: {{ <plan_name>|djstripe_plan_level }}

    Note: Custom settings setup is needed, please see the documentation for details.
    """
    level = -1
    hierarchy_dict = djstripe_settings.PLAN_HIERARCHY

    for config_level in hierarchy_dict.values():
        if name in config_level["plans"]:
            level = config_level["level"]

    return level



# -*- coding: utf-8 -*-
"""
.. module:: dj-stripe.mixins.

   :synopsis: dj-stripe Mixins.

.. moduleauthor:: Daniel Greenfield (@pydanny)

"""

from __future__ import unicode_literals

from django.conf import settings
from django.contrib import messages
from django.shortcuts import redirect

from . import settings as djstripe_settings
from .models import Plan, Customer
from .utils import subscriber_has_active_subscription


class SubscriptionPaymentRequiredMixin(object):
    """
    Check if the subscriber has an active subscription.

    If not, redirect to the subscription page.
    """

    def dispatch(self, request, *args, **kwargs):
        """Redirect user to djstripe:subscribe if account is inactive."""
        if not subscriber_has_active_subscription(djstripe_settings.subscriber_request_callback(request)):
            message = "Your account is inactive. Please renew your subscription"
            messages.info(request, message, fail_silently=True)
            return redirect("djstripe:subscribe")

        return super(SubscriptionPaymentRequiredMixin, self).dispatch(request, *args, **kwargs)


class PaymentsContextMixin(object):
    """Adds plan context to a view."""

    def get_context_data(self, **kwargs):
        """Inject STRIPE_PUBLIC_KEY and plans into context_data."""
        context = super(PaymentsContextMixin, self).get_context_data(**kwargs)
        context.update({
            "STRIPE_PUBLIC_KEY": settings.STRIPE_PUBLIC_KEY,
            "plans": Plan.objects.all(),
        })
        return context


class SubscriptionMixin(PaymentsContextMixin):
    """Adds customer subscription context to a view."""

    def get_context_data(self, *args, **kwargs):
        """Inject is_plans_plural and customer into context_data."""
        context = super(SubscriptionMixin, self).get_context_data(**kwargs)
        context['is_plans_plural'] = Plan.objects.count() > 1
        context['customer'], _created = Customer.get_or_create(
            subscriber=djstripe_settings.subscriber_request_callback(self.request))
        context['subscription'] = context['customer'].subscription
        return context



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-07-05 12:04
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0016_stripe_id_255_length'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='default_source',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='customers',
                                    to='djstripe.StripeSource'),
        ),
        migrations.AlterField(
            model_name='charge',
            name='source',
            field=models.ForeignKey(help_text='The source used for this charge.', null=True,
                                    on_delete=django.db.models.deletion.SET_NULL, related_name='charges',
                                    to='djstripe.StripeSource'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='subscription',
            field=models.ForeignKey(help_text='The subscription that this invoice was prepared for, if any.',
                                    null=True, on_delete=django.db.models.deletion.SET_NULL,
                                    related_name='invoices', to='djstripe.Subscription'),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='plan',
            field=models.ForeignKey(
                help_text='If the invoice item is a proration, the plan of the subscription for \
                which the proration was computed.', null=True, on_delete=django.db.models.deletion.SET_NULL,
                related_name='invoiceitems', to='djstripe.Plan'),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='subscription',
            field=models.ForeignKey(
                help_text='The subscription that this invoice item has been created for, \
                if any.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='invoiceitems',
                to='djstripe.Subscription'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import models, migrations
import django.db.models.deletion

# Can't use the callable because the app registry is not ready yet.
# Really trusting users here... bad idea? probably.
DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL = getattr(settings, "DJSTRIPE_SUBSCRIBER_MODEL", settings.AUTH_USER_MODEL)


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL),
        ('djstripe', '0002_auto_20150122_2000'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='subscriber',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, to=DJSTRIPE_UNSAFE_SUBSCRIBER_MODEL),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import datetime

from django.db import migrations, models
import django.db.models.deletion
from django.utils import timezone

import djstripe.fields


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('djstripe', '0009_auto_20160501_1838'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='stripe_id',
            field=djstripe.fields.StripeIdField(max_length=50, unique=True),
        ),

        migrations.AlterField(
            model_name='event',
            name='customer',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                null=True,
                to='djstripe.Customer',
                help_text='In the event that there is a related customer, this will point to that Customer record'),
        ),
        migrations.AlterField(
            model_name='event',
            name='livemode',
            field=djstripe.fields.StripeNullBooleanField(
                help_text='Null here indicates that the livemode status is unknown or was previously unrecorded. \
                Otherwise, this field indicates whether this record comes from Stripe test mode or live mode \
                operation.', default=False),
        ),
        migrations.AlterField(
            model_name='event',
            name='processed',
            field=models.BooleanField(
                help_text='If validity is performed, webhook event processor(s) may run to take further action on \
                the event. Once these have run, this is set to True.', default=False),
        ),
        migrations.AlterField(
            model_name='event',
            name='stripe_id',
            field=djstripe.fields.StripeIdField(max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name='event',
            name='type',
            field=djstripe.fields.StripeCharField(max_length=250, help_text="Stripe's event description code"),
        ),
        migrations.AlterField(
            model_name='event',
            name='valid',
            field=models.NullBooleanField(
                help_text='Tri-state bool. Null == validity not yet confirmed. Otherwise, this field indicates that \
                this event was checked via stripe api and found to be either authentic (valid=True) or in-authentic \
                (possibly malicious)'),
        ),
        migrations.AlterField(
            model_name='event',
            name='webhook_message',
            field=djstripe.fields.StripeJSONField(
                help_text='data received at webhook. data should be considered to be garbage until validity check is \
                run and valid flag is set'),
        ),

        migrations.AlterField(
            model_name='invoice',
            name='attempt_count',
            field=djstripe.fields.StripeIntegerField(
                help_text='Number of payment attempts made for this invoice, from the perspective of the payment \
                retry schedule. Any payment attempt counts as the first attempt, and subsequently only automatic \
                retries increment the attempt count. In other words, manual payment attempts after the first attempt \
                do not affect the retry schedule.', default=0),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='invoice',
            name='attempted',
            field=djstripe.fields.StripeBooleanField(
                help_text='Whether or not an attempt has been made to pay the invoice. An invoice is not attempted \
                until 1 hour after the ``invoice.created`` webhook, for example, so you might not want to display \
                that invoice as unpaid to your users.', default=False),
        ),

        # Original format is a charge stripe id.... renaming it and creating a new field.
        # The sync in the next migration will take care of filling the charge field.
        migrations.RenameField(
            model_name='invoice',
            old_name='charge',
            new_name='charge_stripe_id'
        ),
        migrations.AlterField(
            model_name='invoice',
            name='charge_stripe_id',
            field=models.CharField(max_length=50, blank=True, null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='charge',
            field=models.OneToOneField(to='djstripe.Charge', on_delete=django.db.models.deletion.CASCADE, related_name='invoice', null=True,
                                       help_text='The latest charge generated for this invoice, if any.'),
        ),

        migrations.AlterField(
            model_name='invoice',
            name='closed',
            field=djstripe.fields.StripeBooleanField(
                help_text="Whether or not the invoice is still trying to collect payment. An invoice is closed if \
                it's either paid or it has been marked closed. A closed invoice will no longer attempt to collect \
                payment.", default=False),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='customer',
            field=models.ForeignKey(related_name='invoices', on_delete=django.db.models.deletion.CASCADE, to='djstripe.Customer',
                                    help_text='The customer associated with this invoice.'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='date',
            field=djstripe.fields.StripeDateTimeField(help_text='The date on the invoice.'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='paid',
            field=djstripe.fields.StripeBooleanField(help_text='The time at which payment will next be attempted.',
                                                     default=False),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='period_end',
            field=djstripe.fields.StripeDateTimeField(
                help_text='End of the usage period during which invoice items were added to this invoice.'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='period_start',
            field=djstripe.fields.StripeDateTimeField(
                help_text='Start of the usage period during which invoice items were added to this invoice.'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='stripe_id',
            field=djstripe.fields.StripeIdField(max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='subtotal',
            field=djstripe.fields.StripeCurrencyField(
                max_digits=7,
                help_text='Only set for upcoming invoices that preview prorations. \
                The time used to calculate prorations.',
                decimal_places=2),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='total',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, decimal_places=2,
                                                      verbose_name='Total after discount.'),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='amount',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, help_text='Amount invoiced.', decimal_places=2),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='currency',
            field=djstripe.fields.StripeCharField(max_length=3, help_text='Three-letter ISO currency code.'),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='description',
            field=djstripe.fields.StripeTextField(blank=True, help_text='A description of this object.', null=True),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='invoice',
            field=models.ForeignKey(related_name='invoiceitems', on_delete=django.db.models.deletion.CASCADE, to='djstripe.Invoice',
                                    help_text='The invoice to which this invoiceitem is attached.'),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='period_end',
            field=djstripe.fields.StripeDateTimeField(
                help_text="Might be the date when this invoiceitem's invoice was sent."),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='period_start',
            field=djstripe.fields.StripeDateTimeField(
                help_text='Might be the date when this invoiceitem was added to the invoice'),
        ),

        # Original format is a charge stripe id.... renaming it and creating a new field.
        # The sync in the next migration will take care of filling the charge field.
        migrations.RenameField(
            model_name='invoiceitem',
            old_name="plan",
            new_name="plan_stripe_id",
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='plan_stripe_id',
            field=models.CharField(max_length=50, blank=True, null=True),
        ),
        migrations.AddField(
            model_name='invoiceitem',
            name='plan',
            field=models.ForeignKey(
                related_name='invoiceitems',
                on_delete=django.db.models.deletion.CASCADE,
                null=True,
                to='djstripe.Plan',
                help_text='If the invoice item is a proration, the plan of the subscription for which the \
                proration was computed.'),
        ),

        migrations.AlterField(
            model_name='invoiceitem',
            name='proration',
            field=djstripe.fields.StripeBooleanField(
                help_text='Whether or not the invoice item was created automatically as a proration adjustment when \
                the customer switched plans.', default=False),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='quantity',
            field=djstripe.fields.StripeIntegerField(
                help_text='If the invoice item is a proration, the quantity of the subscription for which the \
                proration was computed.', null=True),
        ),

        # InvoiceItems stripe_id was the subscription stripe_id... what
        migrations.RenameField(
            model_name='invoiceitem',
            old_name='stripe_id',
            new_name='subscription_stripe_id'
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='subscription_stripe_id',
            field=models.CharField(max_length=50, blank=True, null=True),
        ),
        migrations.AddField(
            model_name='invoiceitem',
            name='stripe_id',
            field=djstripe.fields.StripeIdField(max_length=50, default="unknown", unique=False),
        ),

        migrations.AlterField(
            model_name='plan',
            name='amount',
            field=djstripe.fields.StripeCurrencyField(
                max_digits=7,
                help_text='Amount to be charged on the interval specified.',
                decimal_places=2),
        ),
        migrations.AlterField(
            model_name='plan',
            name='currency',
            field=djstripe.fields.StripeCharField(max_length=3, help_text='Three-letter ISO currency code'),
        ),
        migrations.AlterField(
            model_name='plan',
            name='interval',
            field=djstripe.fields.StripeCharField(
                choices=[('day', 'Day'), ('week', 'Week'), ('month', 'Month'), ('year', 'Year')],
                max_length=5,
                help_text='The frequency with which a subscription should be billed.'),
        ),
        migrations.AlterField(
            model_name='plan',
            name='interval_count',
            field=djstripe.fields.StripeIntegerField(
                help_text='The number of intervals (specified in the interval property) between each \
                subscription billing.', null=True),
        ),
        migrations.AlterField(
            model_name='plan',
            name='name',
            field=djstripe.fields.StripeTextField(
                help_text='Name of the plan, to be displayed on invoices and in the web interface.'),
        ),
        migrations.AlterField(
            model_name='plan',
            name='stripe_id',
            field=djstripe.fields.StripeIdField(max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name='plan',
            name='trial_period_days',
            field=djstripe.fields.StripeIntegerField(
                help_text='Number of trial period days granted when subscribing a customer to this plan. \
                Null if the plan has no trial period.', null=True),
        ),

        migrations.AlterField(
            model_name='subscription',
            name='cancel_at_period_end',
            field=djstripe.fields.StripeBooleanField(
                help_text='If the subscription has been canceled with the ``at_period_end`` flag set to true, \
                ``cancel_at_period_end`` on the subscription will be true. You can use this attribute to determine \
                whether a subscription that has a status of active is scheduled to be canceled at the end of the \
                current period.', default=False),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='canceled_at',
            field=djstripe.fields.StripeDateTimeField(
                help_text='If the subscription has been canceled, the date of that cancellation. \
                If the subscription was canceled with ``cancel_at_period_end``, canceled_at will still reflect the \
                date of the initial cancellation request, not the end of the subscription period when the \
                subscription is automatically moved to a canceled state.', null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='current_period_end',
            field=djstripe.fields.StripeDateTimeField(
                help_text='End of the current period for which the subscription has been invoiced. At the end of \
                this period, a new invoice will be created.', default=datetime.datetime(2100, 1, 1, 0, 0,
                                                                                        tzinfo=timezone.utc)),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='subscription',
            name='current_period_start',
            field=djstripe.fields.StripeDateTimeField(
                help_text='Start of the current period for which the subscription has been invoiced.',
                default=datetime.datetime(2100, 1, 1, 0, 0, tzinfo=timezone.utc)),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='subscription',
            name='customer',
            field=models.ForeignKey(related_name='subscriptions', on_delete=django.db.models.deletion.CASCADE, default=1, to='djstripe.Customer',
                                    help_text='The customer associated with this subscription.'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='subscription',
            name='ended_at',
            field=djstripe.fields.StripeDateTimeField(
                help_text='If the subscription has ended (either because it was canceled or because the customer \
                was switched to a subscription to a new plan), the date the subscription ended.', null=True),
        ),

        # Original format is a charge stripe id.... renaming it and creating a new field.
        #  The sync in the next migration will take care of filling the charge field.
        migrations.RenameField(
            model_name='subscription',
            old_name="plan",
            new_name="plan_stripe_id",
        ),
        migrations.AlterField(
            model_name='subscription',
            name='plan_stripe_id',
            field=models.CharField(max_length=50, blank=True, null=True),
        ),
        migrations.AddField(
            model_name='subscription',
            name='plan',
            field=models.ForeignKey(related_name='subscriptions', on_delete=django.db.models.deletion.CASCADE, default=1, to='djstripe.Plan',
                                    help_text='The plan associated with this subscription.'),
            preserve_default=False,
        ),

        migrations.AlterField(
            model_name='subscription',
            name='quantity',
            field=djstripe.fields.StripeIntegerField(help_text='The quantity applied to this subscription.'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='start',
            field=djstripe.fields.StripeDateTimeField(help_text='Date the subscription started.'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='status',
            field=djstripe.fields.StripeCharField(
                choices=[('trialing', 'Trialing'), ('active', 'Active'), ('past_due', 'Past Due'),
                         ('canceled', 'Canceled'), ('unpaid', 'Unpaid')],
                max_length=8, help_text='The status of this subscription.'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='trial_end',
            field=djstripe.fields.StripeDateTimeField(
                help_text='If the subscription has a trial, the end of that trial.', null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='trial_start',
            field=djstripe.fields.StripeDateTimeField(
                help_text='If the subscription has a trial, the beginning of that trial.', null=True),
        ),

        migrations.AlterField(
            model_name='transfer',
            name='adjustment_count',
            field=djstripe.fields.StripeIntegerField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='adjustment_fees',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, default=None, decimal_places=2, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='adjustment_gross',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, default=None, decimal_places=2, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='amount',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, help_text='The amount transferred',
                                                      decimal_places=2),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='charge_count',
            field=djstripe.fields.StripeIntegerField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='charge_fees',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, default=None, decimal_places=2, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='charge_gross',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, default=None, decimal_places=2, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='collected_fee_count',
            field=djstripe.fields.StripeIntegerField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='collected_fee_gross',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, default=None, decimal_places=2, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='date',
            field=djstripe.fields.StripeDateTimeField(
                help_text="Date the transfer is scheduled to arrive in the bank. \
                This doesn't factor in delays like weekends or bank holidays."),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='description',
            field=djstripe.fields.StripeTextField(blank=True, help_text='A description of this object.', null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='net',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, default=None, decimal_places=2, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='refund_count',
            field=djstripe.fields.StripeIntegerField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='refund_fees',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, default=None, decimal_places=2, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='refund_gross',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, default=None, decimal_places=2, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='status',
            field=djstripe.fields.StripeCharField(
                choices=[('paid', 'Paid'), ('pending', 'Pending'), ('in_transit', 'In Transit'),
                         ('canceled', 'Canceled'), ('failed', 'Failed')],
                max_length=10,
                help_text='The current status of the transfer. A transfer will be pending until it is submitted to \
                the bank, at which point it becomes in_transit. It will then change to paid if the transaction goes \
                through. If it does not go through successfully, its status will change to failed or canceled.'),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='stripe_id',
            field=djstripe.fields.StripeIdField(max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='validation_count',
            field=djstripe.fields.StripeIntegerField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='transfer',
            name='validation_fees',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, default=None, decimal_places=2, null=True),
        ),

        migrations.AlterField(
            model_name='charge',
            name='amount',
            field=djstripe.fields.StripeCurrencyField(help_text='Amount charged.', max_digits=7, default=0,
                                                      decimal_places=2),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='charge',
            name='amount_refunded',
            field=djstripe.fields.StripeCurrencyField(
                help_text='Amount refunded (can be less than the amount attribute on the charge if a partial refund \
                was issued).', max_digits=7, default=0, decimal_places=2),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='charge',
            name='customer',
            field=models.ForeignKey(related_name='charges', on_delete=django.db.models.deletion.CASCADE, to='djstripe.Customer',
                                    help_text='The customer associated with this charge.'),
        ),
        migrations.AlterField(
            model_name='charge',
            name='description',
            field=djstripe.fields.StripeTextField(blank=True, help_text='A description of this object.', null=True),
        ),
        migrations.AlterField(
            model_name='charge',
            name='disputed',
            field=djstripe.fields.StripeBooleanField(help_text='Whether or not this charge is disputed.',
                                                     default=False),
        ),
        migrations.AlterField(
            model_name='charge',
            name='fee',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, decimal_places=2, null=True),
        ),
        migrations.AlterField(
            model_name='charge',
            name='paid',
            field=djstripe.fields.StripeBooleanField(
                help_text='True if the charge succeeded, or was successfully authorized for later capture, \
                False otherwise.', default=False),
        ),
        migrations.AlterField(
            model_name='charge',
            name='refunded',
            field=djstripe.fields.StripeBooleanField(
                help_text='Whether or not the charge has been fully refunded. If the charge is only partially \
                refunded, this attribute will still be false.', default=False),
        ),
        migrations.AlterField(
            model_name='charge',
            name='stripe_id',
            field=djstripe.fields.StripeIdField(max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name='charge',
            name='stripe_timestamp',
            field=djstripe.fields.StripeDateTimeField(help_text='The datetime this object was created in stripe.',
                                                      null=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.admin.

   :synopsis: dj-stripe - Django Administration interface definitions

.. moduleauthor:: Daniel Greenfeld (@pydanny)
.. moduleauthor:: Alex Kavanaugh (@kavdev)
.. moduleauthor:: Lee Skillen (@lskillen)

"""

from django.contrib import admin

from .models import Event, EventProcessingException, Transfer, Charge, Plan
from .models import Invoice, InvoiceItem, Subscription, Customer


class CustomerHasSourceListFilter(admin.SimpleListFilter):
    """A SimpleListFilter used with Customer admin."""

    title = "source presence"
    parameter_name = "has_source"

    def lookups(self, request, model_admin):
        """
        Return a list of tuples.

        The first element in each tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        source: https://docs.djangoproject.com/en/1.10/ref/contrib/admin/#django.contrib.admin.ModelAdmin.list_filter
        """
        return [
            ["yes", "Has Source"],
            ["no", "Does Not Have Source"]
        ]

    def queryset(self, request, queryset):
        """
        Return the filtered queryset based on the value provided in the query string.

        source: https://docs.djangoproject.com/en/1.10/ref/contrib/admin/#django.contrib.admin.ModelAdmin.list_filter
        """
        if self.value() == "yes":
            return queryset.exclude(default_source=None)
        if self.value() == "no":
            return queryset.filter(default_source=None)


class InvoiceCustomerHasSourceListFilter(admin.SimpleListFilter):
    """A SimpleListFilter used with Invoice admin."""

    title = "source presence"
    parameter_name = "has_source"

    def lookups(self, request, model_admin):
        """
        Return a list of tuples.

        The first element in each tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        source: https://docs.djangoproject.com/en/1.10/ref/contrib/admin/#django.contrib.admin.ModelAdmin.list_filter
        """
        return [
            ["yes", "Has Source"],
            ["no", "Does Not Have Source"]
        ]

    def queryset(self, request, queryset):
        """
        Return the filtered queryset based on the value provided in the query string.

        source: https://docs.djangoproject.com/en/1.10/ref/contrib/admin/#django.contrib.admin.ModelAdmin.list_filter
        """
        if self.value() == "yes":
            return queryset.exclude(customer__default_source=None)
        if self.value() == "no":
            return queryset.filter(customer__default_source=None)


class CustomerSubscriptionStatusListFilter(admin.SimpleListFilter):
    """A SimpleListFilter used with Customer admin."""

    title = "subscription status"
    parameter_name = "sub_status"

    def lookups(self, request, model_admin):
        """
        Return a list of tuples.

        The first element in each tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        source: https://docs.djangoproject.com/en/1.10/ref/contrib/admin/#django.contrib.admin.ModelAdmin.list_filter
        """
        statuses = [
            [x, x.replace("_", " ").title()]
            for x in Subscription.objects.all().values_list(
                "status",
                flat=True
            ).distinct()
        ]
        statuses.append(["none", "No Subscription"])
        return statuses

    def queryset(self, request, queryset):
        """
        Return the filtered queryset based on the value provided in the query string.

        source: https://docs.djangoproject.com/en/1.10/ref/contrib/admin/#django.contrib.admin.ModelAdmin.list_filter
        """
        if self.value() is None:
            return queryset.all()
        else:
            return queryset.filter(subscriptions__status=self.value()).distinct()


def send_charge_receipt(modeladmin, request, queryset):
    """Function for sending receipts from the admin if a receipt is not sent for a specific charge."""
    for charge in queryset:
        charge.send_receipt()


admin.site.register(
    Charge,
    list_display=[
        "stripe_id",
        "customer",
        "amount",
        "description",
        "paid",
        "disputed",
        "refunded",
        "fee",
        "receipt_sent",
        "stripe_timestamp",
    ],
    search_fields=[
        "stripe_id",
        "customer__stripe_id",
        "invoice__stripe_id",
    ],
    list_filter=[
        "paid",
        "disputed",
        "refunded",
        "stripe_timestamp",
    ],
    raw_id_fields=[
        "customer",
    ],
    actions=(send_charge_receipt,),
)

admin.site.register(
    EventProcessingException,
    list_display=[
        "message",
        "event",
        "created"
    ],
    search_fields=[
        "message",
        "traceback",
        "data"
    ],
)


def reprocess_events(modeladmin, request, queryset):
    """Re-process the selected webhook events.

    Note that this isn't idempotent, so any side-effects that are produced from
    the event being handled will be multiplied (for example, an event handler
    that sends emails will send duplicates; an event handler that adds 1 to a
    total count will be a count higher than it was, etc.)

    There aren't any event handlers with adverse side-effects built within
    dj-stripe, but there might be within your own event handlers, third-party
    plugins, contrib code, etc.
    """
    processed = 0
    for event in queryset:
        if event.process(force=True):
            processed += 1

    modeladmin.message_user(
        request,
        "{processed}/{total} event(s) successfully re-processed.".format(
            processed=processed, total=queryset.count()))


reprocess_events.short_description = "Re-process selected webhook events"


admin.site.register(
    Event,
    raw_id_fields=["customer"],
    list_display=[
        "stripe_id",
        "type",
        "livemode",
        "valid",
        "processed",
        "stripe_timestamp"
    ],
    list_filter=[
        "type",
        "created",
        "valid",
        "processed"
    ],
    search_fields=[
        "stripe_id",
        "customer__stripe_id",
        "validated_message"
    ],
    actions=[
        reprocess_events
    ],
)


class SubscriptionInline(admin.TabularInline):
    """A TabularInline for use models.Subscription."""

    model = Subscription


def subscription_status(customer):
    """
    Return a string representation of the customer's subscription status.

    If the customer does not have a subscription, an empty string is returned.
    """
    if customer.subscription:
        return customer.subscription.status
    else:
        return ""


subscription_status.short_description = "Subscription Status"


def cancel_subscription(modeladmin, request, queryset):
    """Cancel a subscription."""
    for subscription in queryset:
        subscription.cancel()


cancel_subscription.short_description = "Cancel selected subscriptions"

admin.site.register(
    Subscription,
    raw_id_fields=[
        "customer",
        "plan",
    ],
    list_display=[
        "stripe_id",
        "status",
        "stripe_timestamp",
    ],
    list_filter=[
        "status",
    ],
    search_fields=[
        "stripe_id",
    ],
    actions=[cancel_subscription]
)


admin.site.register(
    Customer,
    raw_id_fields=["subscriber"],
    list_display=[
        "stripe_id",
        "subscriber",
        subscription_status,
        "stripe_timestamp"
    ],
    list_filter=[
        CustomerHasSourceListFilter,
        CustomerSubscriptionStatusListFilter
    ],
    search_fields=[
        "stripe_id"
    ],
    inlines=[SubscriptionInline]
)


class InvoiceItemInline(admin.TabularInline):
    """A TabularInline for use models.InvoiceItem."""

    model = InvoiceItem


def customer_has_source(obj):
    """Return True if the customer has a source attached to its account."""
    return obj.customer.default_source is not None


customer_has_source.short_description = "Customer Has Source"


def customer_email(obj):
    """Return a string representation of the customer's email."""
    return str(obj.customer.subscriber.email)


customer_email.short_description = "Customer"


admin.site.register(
    Invoice,
    raw_id_fields=["customer"],
    readonly_fields=('stripe_timestamp',),
    list_display=[
        "stripe_id",
        "paid",
        "forgiven",
        "closed",
        customer_email,
        customer_has_source,
        "period_start",
        "period_end",
        "subtotal",
        "total",
        "stripe_timestamp"
    ],
    search_fields=[
        "stripe_id",
        "customer__stripe_id"
    ],
    list_filter=[
        InvoiceCustomerHasSourceListFilter,
        "paid",
        "forgiven",
        "closed",
        "attempted",
        "attempt_count",
        "stripe_timestamp",
        "date",
        "period_end",
        "total"
    ],
    inlines=[InvoiceItemInline]
)


admin.site.register(
    Transfer,
    list_display=[
        "stripe_id",
        "amount",
        "status",
        "date",
        "description",
        "stripe_timestamp"
    ],
    search_fields=[
        "stripe_id",
    ]
)


class PlanAdmin(admin.ModelAdmin):
    """An Admin for use with models.Plan."""

    def save_model(self, request, obj, form, change):
        """Update or create objects using our custom methods that sync with Stripe."""
        if change:
            obj.update_name()

        else:
            Plan.get_or_create(**form.cleaned_data)

    def get_readonly_fields(self, request, obj=None):
        """Return extra readonly_fields."""
        readonly_fields = list(self.readonly_fields)
        if obj:
            readonly_fields.extend([
                'stripe_id',
                'amount',
                'currency',
                'interval',
                'interval_count',
                'trial_period_days'])

        return readonly_fields


admin.site.register(Plan, PlanAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-07 08:13
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations
import djstripe.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0020_auto_20161229_0041'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscription',
            name='application_fee_percent',
            field=djstripe.fields.StripePercentField(decimal_places=2, help_text="A positive decimal that represents the fee percentage of the subscription invoice amount that will be transferred to the application owner's Stripe account each billing period.", max_digits=5, null=True, validators=[django.core.validators.MinValueValidator(1.0), django.core.validators.MaxValueValidator(100.0)]),
        ),
    ]
